<?php

namespace FiloBlu\Refilo\Model\Indexer\Core;

use FiloBlu\Refilo\Model\Adapter\Elastic;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterfaceFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Mview\ActionInterface;

abstract class AbstractElasticIndexer implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{
    /**
     * @var SqlToMongoAdapterInterface
     */
    protected $sqlToMongoAdapter;
    /**
     * @var Elastic
     */
    protected $elastic;
    /**
     * @var QueryProviderInterfaceFactory
     */
    protected $queryProviderFactory;
    /**
     * @var string
     */
    private $databaseConfigurationKey;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * AbstractMongoIndexer constructor.
     * @param SqlToMongoAdapterInterface $sqlToMongoAdapter
     * @param QueryProviderInterfaceFactory $queryProviderFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param Elastic $elastic
     * @param string $databaseConfigurationKey
     */
    public function __construct(
        SqlToMongoAdapterInterface $sqlToMongoAdapter,
        QueryProviderInterfaceFactory $queryProviderFactory,
        ScopeConfigInterface $scopeConfig,
        Elastic $elastic,
        string $databaseConfigurationKey
    ) {
        $this->sqlToMongoAdapter = $sqlToMongoAdapter;
        $this->elastic = $elastic;
        $this->queryProviderFactory = $queryProviderFactory;
        $this->databaseConfigurationKey = $databaseConfigurationKey;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @inheritDoc
     */
    public function executeFull()
    {
        $this->performAction();
    }

    protected function performAction($ids = null)
    {
        /** @var QueryProviderInterface $provider */
        $provider = $this->queryProviderFactory->create();
        if ($ids !== null) {
            $provider->filter($ids);
        }

        $result = $this->sqlToMongoAdapter->adaptFromSql($provider);
        $this->elastic->update(
            $this->getBaseKey(),
            $result,
            $this->getTargetMongoCollection(),
            $this->getInsertMode(),
            $this->getMongoDb()
        );
    }

    /**
     * @return string
     */
    abstract public function getBaseKey(): string;

    /**
     * @return string
     */
    abstract public function getTargetMongoCollection(): string;

    /**
     * @return bool
     */
    abstract public function getInsertMode(): bool;

    /**
     * @inheritDoc
     */
    public function executeRow($id)
    {
        $this->executeList([$id]);
    }

    /**
     * @inheritDoc
     */
    public function executeList(array $ids)
    {
        $this->performAction($ids);
    }

    /**
     * @inheritDoc
     */
    public function execute($ids)
    {
        $this->executeList($ids);
    }

    /**
     * @return mixed
     */
    public function getMongoDb()
    {
        return $this->scopeConfig->getValue($this->databaseConfigurationKey);
    }
}
