<?php

namespace FiloBlu\Refilo\Model\Indexer\Provider;

use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Magento\Framework\App\ResourceConnection;

class CategoriesProvider implements QueryProviderInterface
{
    /** @var null */
    protected $ids = null;
    /** @var int */
    protected $website;
    /** @var ResourceConnection */
    private $resourceConnection;
    /** @var array */
    private $bindings = [];

    /**
     * PriceProvider constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
    }

    public function getQuery(): string
    {
        $connection = $this->resourceConnection->getConnection();

        $select = $connection->select();
        $select->from(
            [
                'ccp' => 'catalog_category_product'
            ],
            [
                'categories_ids' => new \Zend_Db_Expr('GROUP_CONCAT(`path` SEPARATOR \'/\')'),
                '_id' => 'product_id'
            ]
        )->join(
            [
                'cpe' => 'catalog_product_entity'
            ],
            'ccp.product_id = cpe.row_id',
            []
        )->join(
            [
                'cce' => 'catalog_category_entity'
            ],
            'ccp.category_id = cce.row_id',
            []
        )->group('product_id');

        if ($this->ids !== null) {
            $select->where('product_id IN (' . implode(',', $this->ids) . ')');
        }

        return $select->__toString();
    }

    public function getBindings(): array
    {
        return $this->bindings;
    }

    public function filter(array $ids): QueryProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    public function website(int $website_id): QueryProviderInterface
    {
        $this->website = $website_id;
        return $this;
    }

    public function store(int $store_id): QueryProviderInterface
    {
        return $this;
    }
}
